package auth

import (
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestParseProtocolVersion(t *testing.T) {
	for _, tt := range []struct {
		initial  string
		expected *ProtocolVersion
		err      string
	}{
		{"3.0", &ProtocolVersion{3, 0}, ""},
		{"3.122", &ProtocolVersion{3, 122}, ""},
		{"3.1.2", nil, "Invalid protocol version: 3.1.2"},
		{"3._.2", nil, "Invalid protocol version: 3._.2"},
		{"_.2", nil, "Invalid protocol version: _.2. '_' is not a valid number"},
		{"2._", nil, "Invalid protocol version: 2._. '_' is not a valid number"},
	} {
		t.Run(tt.initial, func(t *testing.T) {
			version, err := ParseProtocolVersion(tt.initial)
			assert.Equal(t, tt.expected, version)
			if err != nil || tt.err != "" {
				assert.Equal(t, tt.err, err.Error())
			}
		})
	}
}

func TestParseClientInfo(t *testing.T) {
	for _, tt := range []struct {
		initial  string
		expected *ClientInfo
		err      string
	}{
		{"@3.0", &ClientInfo{"", ProtocolVersion{3, 0}, "", ""}, ""},
		{"zou@3.0", &ClientInfo{"zou", ProtocolVersion{3, 0}, "", ""}, ""},
		{"@3.4(xbus-ctl)", &ClientInfo{"", ProtocolVersion{3, 4}, "xbus-ctl", ""}, ""},
		{"@3.4(xbus-ctl@3.4.12-rc.4)", &ClientInfo{"", ProtocolVersion{3, 4}, "xbus-ctl", "3.4.12-rc.4"}, ""},
		{"@3.4(@3.4.12-rc.4)", &ClientInfo{"", ProtocolVersion{3, 4}, "", "3.4.12-rc.4"}, ""},
		{"3.0", nil, "client info should match '([^@]*)@([0-9]*\\.[0-9]*)(\\(([^@]*)(@([^\\)]*))?\\))?' - got '3.0'"},
	} {
		t.Run(tt.initial, func(t *testing.T) {
			info, err := ParseClientInfo(tt.initial)
			assert.Equal(t, tt.expected, info)
			if err == nil && tt.err != "" {
				t.Errorf("Should get error: %s, got nil", tt.err)
			} else if err != nil || tt.err != "" {
				assert.Equal(t, tt.err, err.Error())
			}
		})
	}
}
