// Copyright 2016 Orus.io. All rights reserved.

package cmd

import (
	"io/ioutil"
	"os"

	"github.com/spf13/cobra"
	"github.com/spf13/viper"

	"bitbucket.org/orus-io/go-xbus/tools"
)

// keyCmd represents the key command.
var keyCmd = &cobra.Command{
	Use:              "key",
	Short:            "Manage private keys used by this program.",
	Long:             "Manage private keys used by this program.",
	PersistentPreRun: preRunCheckCertCertDir,
}

// keyGenerateCmd represents the key generation sub-command.
var keyGenerateCmd = &cobra.Command{
	Use:   "generate",
	Short: "Generate private keys.",
	Long: `Generate private keys:

- ServerCA.key: For the server CA (the one clients connecting to Xbus will
  check).

- ClientCA.key: For the client CA (the one clients connecting to Xbus will have
  to have been signed by).`,
	Run: generateKeys,
}

// generateKeys parses arguments to generate private keys.
func generateKeys(cmd *cobra.Command, args []string) {
	certDir := viper.GetString("cert-dir")
	os.Exit(doGenerateKeys(certDir))
}

// doGenerateKeys generates private keys (see the command description for more information).
func doGenerateKeys(certDir string) int {
	generated := genKey(certDir+"/ServerCA.key") && genKey(certDir+"/ClientCA.key")
	if !generated {
		return 1
	}
	return 0
}

func init() {
	RootCmd.AddCommand(keyCmd)

	keyCmd.AddCommand(keyGenerateCmd)
}

// genKey is a helper that creates a private key and saves it into a file.
func genKey(keyPath string) (generated bool) {
	privKeyData, err := tools.CreatePrivateKey()
	if err != nil {
		Logger.Errorf("Error generating the key: %s", err)
		return false
	}

	if err := ioutil.WriteFile(keyPath, privKeyData, 0644); err != nil {
		Logger.Errorf("Error writing the key to %q: %s", keyPath, err)
		return false
	}

	Logger.Noticef("Successfully generated %q.", keyPath)
	return true
}
