package fullstack

import (
	"testing"
	"time"

	"bitbucket.org/orus-io/go-xbus/api"
	raven "github.com/getsentry/raven-go"
	"github.com/stretchr/testify/assert"
)

type TestTransport struct {
	packets []*raven.Packet
}

// Send stores the packet for later inspection
func (tt *TestTransport) Send(url, authHeader string, packet *raven.Packet) error {
	tt.packets = append(tt.packets, packet)
	return nil
}

func TestRaven(t *testing.T) {
	t.Run("CaptureProcessERROR", func(t *testing.T) {
		ft := InitFullstack(t, true)
		if ft == nil {
			return
		}
		defer ft.Shutdown()

		transport := &TestTransport{}

		ft.Server.GetRavenClient().Transport = transport

		ft.CreateRole(api.Actor_EMITTER, "anyemitter")
		_, emitter := ft.CreateEmitter("bob", []string{"anyemitter"})

		_, err := emitter.SendItems(nil, api.DefaultOutput, "evt1", []api.Item{[]byte("item1")})

		if err == nil {
			t.Fatal("Should get a 'no matching pipeline' error")
		}
		assert.Equal(t, "No matching pipeline", err.Error())

		// Make sure the packet has time to be sent
		time.Sleep(100 * time.Millisecond)

		assert.Equal(t, 1, len(transport.packets))

		if len(transport.packets) == 0 {
			return
		}

		packet := transport.packets[0]
		assert.Regexp(t, "Process .* failed for the following reason: No matching pipeline", packet.Message)

	})
}
